Package: apollo
Type: Package
Title: Tools for Choice Model Estimation and Application
Version: 0.3.8
Authors@R: c(
    person("Stephane", "Hess", email="S.Hess@leeds.ac.uk", role=c("aut","cre")),
    person("David", "Palma", email="D.Palma@leeds.ac.uk", role="aut"),
    person("Thomas", "Hancock", email="T.O.Hancock@leeds.ac.uk", role="ctb"))
Description: Choice models are a widely used technique across numerous scientific disciplines. The Apollo package is a very flexible tool for the estimation and application 
    of choice models in R. Users are able to write their own 
    model functions or use a mix of already available ones. Random heterogeneity, 
    both continuous and discrete and at the level of individuals and 
    choices, can be incorporated for all models. There is support for both standalone 
    models and hybrid model structures.  Both classical 
    and Bayesian estimation is available, and multiple discrete 
    continuous models are covered in addition to discrete choice. 
    Multi-threading processing is supported for estimation and a large
    number of pre and post-estimation routines, including for computing posterior
    (individual-level) distributions are available. 
    For examples, a manual, and a support forum, visit 
    <https://www.ApolloChoiceModelling.com>. For more information on choice 
    models see Train, K. (2009) <isbn:978-0-521-74738-7> and Hess, 
    S. & Daly, A.J. (2014) <isbn:978-1-781-00314-5> for an overview 
    of the field.
License: GPL-2
URL: https://www.ApolloChoiceModelling.com
BugReports: https://www.ApolloChoiceModelling.com/forum/
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.3.0), stats, utils
Imports: Rcpp (>= 1.0.0), maxLik, mnormt, mvtnorm, graphics,
        randtoolbox, numDeriv, parallel, Deriv, matrixStats, coda,
        tibble, stringr, bgw (>= 0.1.4), cli, Rsolnp, rstudioapi,
        MCMCpack
LinkingTo: Rcpp, RcppArmadillo, RcppEigen
Suggests: knitr, rmarkdown, testthat
VignetteBuilder: knitr
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-05-17 19:45:55 UTC; stephane.hess
Author: Stephane Hess [aut, cre],
  David Palma [aut],
  Thomas Hancock [ctb]
Maintainer: Stephane Hess <S.Hess@leeds.ac.uk>
Repository: CRAN
Date/Publication: 2026-05-17 22:40:16 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-05-18 00:42:33 UTC; unix
Archs: apollo.so.dSYM
