# Generated by roxygen2: do not edit by hand

S3method(print,apollo)
S3method(summary,apollo)
export(apollo_addCovariance)
export(apollo_attach)
export(apollo_avgInterDraws)
export(apollo_avgIntraDraws)
export(apollo_basTest)
export(apollo_bootstrap)
export(apollo_checkArguments)
export(apollo_choiceAnalysis)
export(apollo_classAlloc)
export(apollo_cnl)
export(apollo_cnl2)
export(apollo_combineModels)
export(apollo_combineResults)
export(apollo_compareInputs)
export(apollo_conditionals)
export(apollo_dVdB)
export(apollo_dVdBOld)
export(apollo_deltaMethod)
export(apollo_detach)
export(apollo_dft)
export(apollo_diagnostics)
export(apollo_el)
export(apollo_emdc)
export(apollo_emdc1)
export(apollo_emdc2)
export(apollo_estimate)
export(apollo_estimateHB)
export(apollo_expandLoop)
export(apollo_firstRow)
export(apollo_fitsTest)
export(apollo_fmnl)
export(apollo_fnl)
export(apollo_initialise)
export(apollo_insertComponentName)
export(apollo_insertFunc)
export(apollo_insertOLList)
export(apollo_insertRRMQuotes)
export(apollo_insertRows)
export(apollo_insertScaling)
export(apollo_keepRows)
export(apollo_lc)
export(apollo_lcConditionals)
export(apollo_lcEM)
export(apollo_lcUnconditionals)
export(apollo_llCalc)
export(apollo_loadModel)
export(apollo_longToWide)
export(apollo_lrTest)
export(apollo_makeCluster)
export(apollo_makeDraws)
export(apollo_makeGrad)
export(apollo_makeHessian)
export(apollo_makeLogLike)
export(apollo_mdcev)
export(apollo_mdcev2)
export(apollo_mdcnev)
export(apollo_mixConditionals)
export(apollo_mixEM)
export(apollo_mixUnconditionals)
export(apollo_mlhs)
export(apollo_mnl)
export(apollo_modelOutput)
export(apollo_modifyUserDefFunc)
export(apollo_nl)
export(apollo_normalDensity)
export(apollo_ol)
export(apollo_op)
export(apollo_outOfSample)
export(apollo_ownModel)
export(apollo_panelProd)
export(apollo_preEstimate)
export(apollo_prediction)
export(apollo_prepareProb)
export(apollo_preprocess)
export(apollo_print)
export(apollo_readBeta)
export(apollo_rrm)
export(apollo_saveOutput)
export(apollo_searchStart)
export(apollo_setRows)
export(apollo_setWorkDir)
export(apollo_sharesTest)
export(apollo_sink)
export(apollo_speedTest)
export(apollo_tobit)
export(apollo_unconditionals)
export(apollo_validate)
export(apollo_validateControl)
export(apollo_validateData)
export(apollo_validateHBControl)
export(apollo_validateInputs)
export(apollo_varList)
export(apollo_varcov)
export(apollo_weighting)
export(apollo_writeF12)
export(apollo_writeTheta)
export(aux_validateRows)
import(Rcpp)
importFrom(MCMCpack,rinvgamma)
importFrom(MCMCpack,riwish)
importFrom(Rsolnp,solnp)
importFrom(bgw,bgw_mle)
importFrom(coda,geweke.diag)
importFrom(grDevices,dev.flush)
importFrom(grDevices,graphics.off)
importFrom(grDevices,rainbow)
importFrom(graphics,axis)
importFrom(graphics,legend)
importFrom(graphics,matplot)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,segments)
importFrom(graphics,title)
importFrom(matrixStats,rowCounts)
importFrom(maxLik,maxLik)
importFrom(mnormt,pmnorm)
importFrom(mnormt,rmnorm)
importFrom(mvtnorm,rmvnorm)
importFrom(numDeriv,grad)
importFrom(numDeriv,hessian)
importFrom(parallel,clusterCall)
importFrom(parallel,stopCluster)
importFrom(randtoolbox,halton)
importFrom(randtoolbox,sobol)
importFrom(rstudioapi,getActiveDocumentContext)
importFrom(stats,aggregate)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,density)
importFrom(stats,dnorm)
importFrom(stats,pnorm)
importFrom(stats,printCoefmat)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(stringr,str_replace_all)
importFrom(tibble,is_tibble)
importFrom(utils,capture.output)
importFrom(utils,installed.packages)
importFrom(utils,lsf.str)
importFrom(utils,read.csv)
importFrom(utils,stack)
importFrom(utils,write.table)
useDynLib(apollo, .registration=TRUE)
