# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

RCPPinv <- function(M) {
    .Call(`_apollo_RCPPinv`, M)
}

RCPPpower <- function(S, Sn) {
    .Call(`_apollo_RCPPpower`, S, Sn)
}

RCPPeta <- function(S, Sn, o, mu, P0) {
    .Call(`_apollo_RCPPeta`, S, Sn, o, mu, P0)
}

RCPPZ <- function(S) {
    .Call(`_apollo_RCPPZ`, S)
}

RCPPomega <- function(Z, Zn, o, o2, phiB) {
    .Call(`_apollo_RCPPomega`, Z, Zn, o, o2, phiB)
}

RCPPphiB <- function(phi, o) {
    .Call(`_apollo_RCPPphiB`, phi, o)
}

RCPPphi <- function(C, M, psi, s) {
    .Call(`_apollo_RCPPphi`, C, M, psi, s)
}

DFTprob <- function(attr, P0, wts, choice, erv, ts, phi1, phi2) {
    .Call(`_apollo_DFTprob`, attr, P0, wts, choice, erv, ts, phi1, phi2)
}

cpp_pmnorm <- function(x, y, f) {
    .Call(`_apollo_cpp_pmnorm`, x, y, f)
}

